# debtagshw: lib to detect what hardware tags apply to the current system
#
# Copyright (C) 2012  Canonical
#
# Author:
#  Michael Vogt <mvo@ubuntu.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import logging

LOG=logging.getLogger(__name__)

class OpenGLError(Exception):
    pass

class OpenGL(object):

    # from /usr/include/GL/glx.h
    GLX_VENDOR=1
    GLX_VERSION=2
    GLX_EXTENSIONS=3

    # for the visinfo 
    GLX_RGBA = 4
    GLX_RED_SIZE = 8
    GLX_GREEN_SIZE = 9
    GLX_BLUE_SIZE = 10
    GLX_DOUBLEBUFFER = 5

    # GL, from /usr/include/GL/gl.h
    GL_VENDOR = 0x1F00
    GL_RENDERER = 0x1F01
    GL_VERSION = 0x1F02
    GL_EXTENSIONS = 0x1F03

    def __init__(self, direct_rendering=True):
        self.direct = direct_rendering

    def opengl_supported(self):
        from ctypes import cdll, c_char, c_char_p, c_int, byref
        # load stuff
        x11 = cdll.LoadLibrary("libX11.so")
        glx = cdll.LoadLibrary("libGL.so")
        # get display
        display = x11.XOpenDisplay("")
        if not display:
            return False
        # get extenstion
        dummy1 = c_char()
        dummy2 = c_char()
        res = glx.glXQueryExtension(display, byref(dummy1), byref(dummy2) )
        if not res:
            return False
        # get screen and window
        screen = x11.XDefaultScreen(display)
        root = x11.XRootWindow(display, screen)
        # create a window to make glGetString work
        attribSingleType = c_int * 8
        attribSingle = attribSingleType(
            self.GLX_RGBA,
            self.GLX_RED_SIZE, 1,
            self.GLX_GREEN_SIZE, 1,
            self.GLX_BLUE_SIZE, 1,
            0)
        visinfo = glx.glXChooseVisual(display, 0, attribSingle)
        if not visinfo:
            attribDoubleType = c_int * 9
            attribDouble = attribDoubleType(
                self.GLX_RGBA,
                self.GLX_RED_SIZE, 1,
                self.GLX_GREEN_SIZE, 1,
                self.GLX_BLUE_SIZE, 1,
                self.GLX_DOUBLEBUFFER,
                0)
            visinfo = glx.glXChooseVisual(display, 0, attribDouble)
        if not visinfo:
            raise OpenGLError("Can not get visinfo")
        # create context etc 
        context = glx.glXCreateContext (display, visinfo, None, self.direct)
        if not context:
            raise OpenGLError("Can not create glx context")
        # make root current
        glx.glXMakeCurrent(display, root, context)
        # and get the actual useful gl data
        glx.glGetString.restype = c_char_p
        vendor = glx.glGetString(self.GL_VENDOR)
        renderer = glx.glGetString(self.GL_RENDERER)
        version = glx.glGetString(self.GL_VERSION)
        extensions = glx.glGetString(self.GL_EXTENSIONS)
        LOG.info("gl vendor: %s" % vendor)
        LOG.info("gl renderer: %s" % renderer)
        LOG.info("gl version: %s" % version)
        LOG.debug("gl extenstions: %s" % extensions)
        # from unity_support_test.c
        for nono in ["Software Rasterizer",
                     "Mesa X11",
                     "on softpipe"]:
            if nono in renderer:
                return False
        # stuff looks ok
        return True

def run_check():
    try:
        oh = OpenGL()
        return oh.opengl_supported()
    except OSError:
        return None

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    o = OpenGL()
    print "opengl_supported: ", o.opengl_supported()

